@@ -1,7 +1,7 @@
 (**************************************************************************)
 (*  The CDuce compiler                                                    *)
 (*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
-(*  Copyright CNRS,INRIA, 2003,2004,2005,2006,2007 (see LICENSE for details)  *)
+(*  Copyright CNRS,INRIA, 2003-2009 (see LICENSE for details)             *)
 (**************************************************************************)
 
 #load "q_MLast.cmo";;
@@ -38,11 +38,21 @@ let id s =
       let j = String.index_from s i '.' in
       <:ident< $uid:String.sub s i (j - i)$.$aux (j+1)$ >>
     with Not_found ->
-      <:ident< $lid:String.sub s i (String.length s - i)$ >>
+      <:ident< $uid:String.sub s i (String.length s - i)$ >>
   in
 (*  Printf.eprintf "*** %S\n" s; *)
   aux 0
 
+let consId s =
+  let rec aux i : Ast.ident =
+    try
+      let j = String.index_from s i '.' in
+      <:ident< $uid:String.sub s i (j - i)$.$aux (j+1)$ >>
+    with Not_found ->
+      <:ident< $uid:String.sub s i (String.length s - i)$ >>
+  in
+  aux 0
+
 let rec typ t =
   try IntHash.find memo_typ t.uid
   with Not_found ->
@@ -399,9 +409,14 @@ and to_ml_descr e = function
 		 let vars = mk_vars tl in
 		 let x = mk_var () in
 		 <:match_case< ($str: String.escaped lab$, Some $lid:x$) ->
-		 $matches <:expr< $lid:x$ >> 
-		         <:expr< $id:id (p ^ lab)$ $tuple_to_ml tl vars$ >> 
-		   vars$ >>
+   	       $ matches
+               <:expr< $lid:x$ >> (
+                      List.fold_left
+                        (fun x (t, id) ->
+                          Ast.ExApp(_loc, x, <:expr<$to_ml <:expr< $lid:id$ >> t$>>))
+                 <:expr< $id:consId (p ^ lab)$ >>
+                        (List.combine tl vars))
+                 vars $ >>
 	  ) l in
       let cases = cases @ [ <:match_case< _ -> assert False >> ] in
       pmatch <:expr< Value.get_variant $e$ >> cases
